==> seplib-6.3.5.tar.gz: seplib-6.3.5/seplib_prog/seis/filter/Pef.f90 <==
!!$=head1 NAME
!!$
!!$Pef - Estimate PEF in N dimensions
!!$
!!$=head1 SYNOPSIS
!!$
!!$Pef < data.H a= [center=] [gap=] [maskin=] [maskout=] > pef.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$Estimates PEF by least squares, using helix convolution. 
!!$Can ignore missing data 
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history
!!$
!!$B<integer array> I<n> - reads I<n1>, I<n2>, I<n3>, etc.
!!$
!!$=item from par
!!$
!!$B<integer array> I<a>= - filter size
!!$
!!$B<integer> I<niter>=2*prod(a) (optional) - number of 
!!$PEF iterations
!!$
!!$B<integer array> I<center>=a/2+1 (optional) - zero-lag position (filter centering)
!!$
!!$B<integer array> I<gap>=0 (optional) - filter gap
!!$
!!$B<filename> I<maskin, maskout> (optional) - data selectors 
!!$
!!$=item to history
!!$
!!$B<integer array> I<lag> - comma separated list of filter lags
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$F<gee/mda/hole90/>, F<gee/mda/miss390/>, F<gee/mda/morgan/>, 
!!$F<gee/mda/seab90/>, F<gee/pch/signoi90/>
!!$
!!$=head1 MODULES
!!$
!!$F<shape.r90>, F<bound.r90>, F<misinput.r90>, F<pef.r90>, 
!!$F<compress.r90>, F<print.r90>, F<helix.r90>
!!$
!!$=head1 SEE ALSO
!!$
!!$L<MSPef>, L<Fillmiss>, L<Helicon>, L<Decon>
!!$
!!$=cut
!%
program Pef3
  use sep
  use helix
  use createhelixmod
  use bound
  use misinput
  use pef
  use compress
  use print
 
  implicit none
  integer, dimension (:), allocatable :: n, n0, a, center, gap
  integer                             :: dim, n123, i, niter, na
  real,    dimension (:), pointer     :: dd, kk
  type( filter)                       :: aa   
  character (len = 6)                 :: varname

  call sep_init ("SOURCE")
call sep_add_doc_line("NAME")
call sep_add_doc_line("    Pef - Estimate PEF in N dimensions")
call sep_add_doc_line("")
call sep_add_doc_line("SYNOPSIS")
call sep_add_doc_line("    Pef < data.H a= [center=] [gap=] [maskin=] [maskout=] > pef.H")
call sep_add_doc_line("")
call sep_add_doc_line("DESCRIPTION")
call sep_add_doc_line("    Estimates PEF by least squares, using helix convolution. Can ignore")
call sep_add_doc_line("    missing data")
call sep_add_doc_line("")
call sep_add_doc_line("PARAMETERS")
call sep_add_doc_line("    from history")
call sep_add_doc_line("       integer array *n* - reads *n1*, *n2*, *n3*, etc.")
call sep_add_doc_line("")
call sep_add_doc_line("    from par")
call sep_add_doc_line("       integer array *a*= - filter size")
call sep_add_doc_line("")
call sep_add_doc_line("       integer *niter*=2*prod(a) (optional) - number of PEF iterations")
call sep_add_doc_line("")
call sep_add_doc_line("       integer array *center*=a/2+1 (optional) - zero-lag position (filter")
call sep_add_doc_line("       centering)")
call sep_add_doc_line("")
call sep_add_doc_line("       integer array *gap*=0 (optional) - filter gap")
call sep_add_doc_line("")
call sep_add_doc_line("       filename *maskin, maskout* (optional) - data selectors")
call sep_add_doc_line("")
call sep_add_doc_line("    to history")
call sep_add_doc_line("       integer array *lag* - comma separated list of filter lags")
call sep_add_doc_line("")
call sep_add_doc_line("EXAMPLES")
call sep_add_doc_line("    gee/mda/hole90/, gee/mda/miss390/, gee/mda/morgan/, gee/mda/seab90/,")
call sep_add_doc_line("    gee/pch/signoi90/")
call sep_add_doc_line("")
call sep_add_doc_line("MODULES")
call sep_add_doc_line("    shape.r90, bound.r90, misinput.r90, pef.r90, compress.r90, print.r90,")
call sep_add_doc_line("    helix.r90")
call sep_add_doc_line("")
call sep_add_doc_line("SEE ALSO")
call sep_add_doc_line("    MSPef, Fillmiss, Helicon, Decon")
call sep_add_doc_line("")

  dim = sep_dimension ()
  allocate (n (dim), n0 (dim), center (dim), a (dim), gap (dim))
  call from_history (n)
  call to_history ("n",n)
  a=1; n0=1; center=1; gap=1
  call from_param ("a", a)
  center = 1
  where (a (2:) > 1) center (:dim-1) = a(:dim-1)/2 + 1
  call from_param ("center", center, center)
  call from_param ("na", na, 0)
  if (na == 0) then
     gap = 0
     call from_param ("gap", gap, gap)
     aa = createhelix( n, center, gap, a) ! allocate PEF
     n0 = n
  else
     call allocatehelix (aa, na)
     call from_param ("lag", aa%lag)
     call from_param ("n", n0)
  end if

  n123 = product (n)
  allocate (dd (n123), kk (n123)) 
  call sep_read (dd)
  if (exist_file ("maskin")) then
     call sep_read (kk, "maskin")
  else
     kk = 1.
  end if

  call boundn( n0, n, a, aa) 
  call find_mask( (kk /= 0.), aa)   ! missing data

  if (exist_file ("maskout")) then
     where (aa%mis) 
        kk = 0.
     elsewhere
        kk = 1.
     end where
     do i=1, dim
        call to_history (axisname(i),n(i),"maskout")
     end do
     call to_history ("esize",4,"maskout")
     call sep_write (kk, "maskout")
  end if

  call from_param( "niter", niter, size(aa%lag)*2 )

  call find_pef ( dd, aa, niter)         ! estimate aa
  call resize( aa)                       ! eliminate zeroes
  call printn( n, center, a, aa)         ! print filter

  call to_history ("n1", size ( aa%lag))
  do i=2, dim
     call to_history (axisname(i),1)
  end do
  call to_history ("lag", aa%lag)
!  do i=1, size (aa%lag)
!     if (i < 10) then
!        write (varname,"(a,i1)") "lag",i
!     else if (i < 100) then
!        write (varname,"(a,i2)") "lag",i
!     else
!        write (varname,"(a,i3)") "lag",i
!     end if
!     call to_history_int (varname, aa%lag (i))
!  end do
  call sep_write (aa%flt)
 
  call sep_close ()
  deallocate (dd, kk)
  deallocate (n, n0, center, a, gap)
  call deallocatehelix( aa)
  call exit (0)
end program Pef3

==> seplib-6.5.2.tar.gz: seplib-6.5.2/seplib_prog/seis/filter/Pef.f90 <==
!!$=head1 NAME
!!$
!!$Pef - Estimate PEF in N dimensions
!!$
!!$=head1 SYNOPSIS
!!$
!!$Pef < data.H a= [center=] [gap=] [maskin=] [maskout=] > pef.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$Estimates PEF by least squares, using helix convolution. 
!!$Can ignore missing data 
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history
!!$
!!$B<integer array> I<n> - reads I<n1>, I<n2>, I<n3>, etc.
!!$
!!$=item from par
!!$
!!$B<integer array> I<a>= - filter size
!!$
!!$B<integer> I<niter>=2*prod(a) (optional) - number of 
!!$PEF iterations
!!$
!!$B<integer array> I<center>=a/2+1 (optional) - zero-lag position (filter centering)
!!$
!!$B<integer array> I<gap>=0 (optional) - filter gap
!!$
!!$B<filename> I<maskin, maskout> (optional) - data selectors 
!!$
!!$=item to history
!!$
!!$B<integer array> I<lag> - comma separated list of filter lags
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$F<gee/mda/hole90/>, F<gee/mda/miss390/>, F<gee/mda/morgan/>, 
!!$F<gee/mda/seab90/>, F<gee/pch/signoi90/>
!!$
!!$=head1 MODULES
!!$
!!$F<shape.r90>, F<bound.r90>, F<misinput.r90>, F<pef.r90>, 
!!$F<compress.r90>, F<print.r90>, F<helix.r90>
!!$
!!$=head1 SEE ALSO
!!$
!!$L<MSPef>, L<Fillmiss>, L<Helicon>, L<Decon>
!!$
!!$=cut
!%
program Pef3
  use sep
  use helix
  use createhelixmod
  use bound
  use misinput
  use pef
  use compress
  use print
 
  implicit none
  integer, dimension (:), allocatable :: n, n0, a, center, gap
  integer                             :: dim, n123, i, niter, na
  real,    dimension (:), pointer     :: dd, kk
  type( filter)                       :: aa   
  character (len = 6)                 :: varname

  call sep_init ("/opt/SEP/seplib/seplib_prog/seis/filter/Pef.f90s")
call sep_add_doc_line("NAME")
call sep_add_doc_line("    Pef - Estimate PEF in N dimensions")
call sep_add_doc_line("")
call sep_add_doc_line("SYNOPSIS")
call sep_add_doc_line("    Pef < data.H a= [center=] [gap=] [maskin=] [maskout=] > pef.H")
call sep_add_doc_line("")
call sep_add_doc_line("DESCRIPTION")
call sep_add_doc_line("    Estimates PEF by least squares, using helix convolution. Can ignore")
call sep_add_doc_line("    missing data")
call sep_add_doc_line("")
call sep_add_doc_line("PARAMETERS")
call sep_add_doc_line("    from history")
call sep_add_doc_line("       integer array *n* - reads *n1*, *n2*, *n3*, etc.")
call sep_add_doc_line("")
call sep_add_doc_line("    from par")
call sep_add_doc_line("       integer array *a*= - filter size")
call sep_add_doc_line("")
call sep_add_doc_line("       integer *niter*=2*prod(a) (optional) - number of PEF iterations")
call sep_add_doc_line("")
call sep_add_doc_line("       integer array *center*=a/2+1 (optional) - zero-lag position (filter")
call sep_add_doc_line("       centering)")
call sep_add_doc_line("")
call sep_add_doc_line("       integer array *gap*=0 (optional) - filter gap")
call sep_add_doc_line("")
call sep_add_doc_line("       filename *maskin, maskout* (optional) - data selectors")
call sep_add_doc_line("")
call sep_add_doc_line("    to history")
call sep_add_doc_line("       integer array *lag* - comma separated list of filter lags")
call sep_add_doc_line("")
call sep_add_doc_line("EXAMPLES")
call sep_add_doc_line("    gee/mda/hole90/, gee/mda/miss390/, gee/mda/morgan/, gee/mda/seab90/,")
call sep_add_doc_line("    gee/pch/signoi90/")
call sep_add_doc_line("")
call sep_add_doc_line("MODULES")
call sep_add_doc_line("    shape.r90, bound.r90, misinput.r90, pef.r90, compress.r90, print.r90,")
call sep_add_doc_line("    helix.r90")
call sep_add_doc_line("")
call sep_add_doc_line("SEE ALSO")
call sep_add_doc_line("    MSPef, Fillmiss, Helicon, Decon")
call sep_add_doc_line("")

  dim = sep_dimension ()
  allocate (n (dim), n0 (dim), center (dim), a (dim), gap (dim))
  call from_history (n)
  call to_history ("n",n)
  a=1; n0=1; center=1; gap=1
  call from_param ("a", a)
  center = 1
  where (a (2:) > 1) center (:dim-1) = a(:dim-1)/2 + 1
  call from_param ("center", center, center)
  call from_param ("na", na, 0)
  if (na == 0) then
     gap = 0
     call from_param ("gap", gap, gap)
     aa = createhelix( n, center, gap, a) ! allocate PEF
     n0 = n
  else
     call allocatehelix (aa, na)
     call from_param ("lag", aa%lag)
     call from_param ("n", n0)
  end if

  n123 = product (n)
  allocate (dd (n123), kk (n123)) 
  call sep_read (dd)
  if (exist_file ("maskin")) then
     call sep_read (kk, "maskin")
  else
     kk = 1.
  end if

  call boundn( n0, n, a, aa) 
  call find_mask( (kk /= 0.), aa)   ! missing data

  if (exist_file ("maskout")) then
     where (aa%mis) 
        kk = 0.
     elsewhere
        kk = 1.
     end where
     do i=1, dim
        call to_history (axisname(i),n(i),"maskout")
     end do
     call to_history ("esize",4,"maskout")
     call sep_write (kk, "maskout")
  end if

  call from_param( "niter", niter, size(aa%lag)*2 )

  call find_pef ( dd, aa, niter)         ! estimate aa
  call resize( aa)                       ! eliminate zeroes
  call printn( n, center, a, aa)         ! print filter

  call to_history ("n1", size ( aa%lag))
  do i=2, dim
     call to_history (axisname(i),1)
  end do
  call to_history ("lag", aa%lag)
!  do i=1, size (aa%lag)
!     if (i < 10) then
!        write (varname,"(a,i1)") "lag",i
!     else if (i < 100) then
!        write (varname,"(a,i2)") "lag",i
!     else
!        write (varname,"(a,i3)") "lag",i
!     end if
!     call to_history_int (varname, aa%lag (i))
!  end do
  call sep_write (aa%flt)
 
  call sep_close ()
  deallocate (dd, kk)
  deallocate (n, n0, center, a, gap)
  call deallocatehelix( aa)
  call exit (0)
end program Pef3
